@@ -5,6 +5,8 @@ from . import iostream
 from .data import HeterodynedData, MultiHeterodynedData
 from .likelihood import TargetedPulsarLikelihood
 from .info import *
+from .parfile import PulsarParameters
+from .knope import skyshift
 
 try:
     from ._version import version as __version__
