@@ -3,11 +3,11 @@ Classes providing likelihood functions.
 """
 
 import re
+from copy import deepcopy
 
 import bilby
 import lal
 import numpy as np
-from lalpulsar.PulsarParametersWrapper import PulsarParametersPy
 from numba import jit, types
 from numba.typed import Dict as numbadict
 
@@ -86,30 +86,30 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
 
     # the set of potential non-GR "amplitude" parameters
     NONGR_AMPLITUDE_PARAM = [
-        "PHI01TENSOR",
-        "PHI02TENSOR",
-        "PHI01VECTOR",
-        "PHI02VECTOR",
-        "PHI01SCALAR",
-        "PHI02SCALAR",
-        "PSI1TENSOR",
-        "PSI2TENSOR",
-        "PSI1VECTOR",
-        "PSI2VECTOR",
-        "PSI1SCALAR",
-        "PSI2SCALAR",
-        "H1PLUS",
-        "H2PLUS",
-        "H1CROSS",
-        "H2CROSS",
-        "H1VECTORX",
-        "H2VECTORX",
-        "H1VECTORY",
-        "H2VECTORY",
-        "H1SCALARB",
-        "H2SCALARB",
-        "H1SCALARL",
-        "H2SCALARL",
+        "PHI0TENSOR",
+        "PHI0TENSOR_F",
+        "PHI0VECTOR",
+        "PHI0VECTOR_F",
+        "PHI0SCALAR",
+        "PHI0SCALAR_F",
+        "PSITENSOR",
+        "PSITENSOR_F",
+        "PSIVECTOR",
+        "PSIVECTOR_F",
+        "PSISCALAR",
+        "PSISCALAR_F",
+        "HPLUS",
+        "HPLUS_F",
+        "HCROSS",
+        "HCROSS_F",
+        "HVECTORX",
+        "HVECTORX_F",
+        "HVECTORY",
+        "HVECTORY_F",
+        "HSCALARB",
+        "HSCALARB_F",
+        "HSCALARL",
+        "HSCALARL_F",
     ]
 
     # the set of positional parameters
@@ -156,9 +156,7 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
     # the parameters that are held as vectors
     VECTOR_PARAMS = ["F", "GLEP", "GLPH", "GLF0", "GLF1", "GLF2", "GLF0D", "GLTD", "FB"]
 
-    def __init__(
-        self, data, priors, par=None, det=None, likelihood="studentst", numba=False
-    ):
+    def __init__(self, data, priors, likelihood="studentst", numba=False):
 
         super().__init__(dict())  # initialise likelihood class
 
@@ -208,6 +206,7 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                 not in self.AMPLITUDE_PARAMS
                 + self.BINARY_PARAMS
                 + self.POSITIONAL_PARAMETERS
+                + self.NONGR_AMPLITUDE_PARAM
             ) and not self._is_vector_param(key.upper()):
                 raise ValueError("Unknown parameter '{}' being used!".format(key))
 
@@ -261,15 +260,11 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                 )
             )
             # copy of heterodyned parameters
-            newpar = PulsarParametersPy()
-            for item in het.par.items():
-                newpar[item[0]] = item[1]
+            newpar = deepcopy(het.par)
             self.basepars.append(newpar)
 
-        # if phase evolution is not in the model set the pre-summed products
-        # of the data and antenna patterns
-        if not self.include_phase:
-            self.dot_products()
+        # set the pre-summed products of the data and antenna patterns
+        self.dot_products()
 
     @property
     def likelihood(self):
@@ -483,22 +478,10 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                     # make sure values are floats
                     par[pname.upper()] = float(pval)
 
-                    if pname.upper() in self.SOURCE_AMPLITUDE_PARAMETERS:
-                        # reset waveform parameters (otherwise these can
-                        # potentially be given by the previous value if the
-                        # new value is zero). NOTE: in the future it might be
-                        # better to pass a copy of the PulsarParameters
-                        # structure to the model function, so that this is not
-                        # problem
-                        par["C21"] = 0.0
-                        par["C22"] = 0.0
-                        par["PHI22"] = 0.0
-                        par["PHI21"] = 0.0
-
             # calculate the model
             m = model.model(
-                par,
-                usephase=self.include_phase,
+                deepcopy(par),
+                outputampcoeffs=(not self.include_phase),
                 updateSSB=self.update_ssb,
                 updateBSB=self.include_binary,
                 updateglphase=self.include_glitch,
@@ -546,8 +529,8 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                         mc = m[1]  # tensor cross model component
 
                         summodel = (
-                            prods["TpdotTp"][i] * (mp.real ** 2 + mp.imag ** 2)
-                            + prods["TcdotTc"][i] * (mc.real ** 2 + mc.imag ** 2)
+                            prods["TpdotTp"][i] * (mp.real**2 + mp.imag**2)
+                            + prods["TcdotTc"][i] * (mc.real**2 + mc.imag**2)
                             + 2.0
                             * prods["TpdotTc"][i]
                             * (mp.real * mc.real + mp.imag * mc.imag)
@@ -568,10 +551,10 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                             ml = m[5]
 
                             summodel += (
-                                prods["VxdotVx"][i] * (mx.real ** 2 + mx.imag ** 2)
-                                + prods["VydotVy"][i] * (my.real ** 2 + my.imag ** 2)
-                                + prods["SbdotSb"][i] * (mb.real ** 2 + mb.imag ** 2)
-                                + prods["SldotSl"][i] * (ml.real ** 2 + ml.imag ** 2)
+                                prods["VxdotVx"][i] * (mx.real**2 + mx.imag**2)
+                                + prods["VydotVy"][i] * (my.real**2 + my.imag**2)
+                                + prods["SbdotSb"][i] * (mb.real**2 + mb.imag**2)
+                                + prods["SldotSl"][i] * (ml.real**2 + ml.imag**2)
                                 + 2.0
                                 * (
                                     prods["TpdotVx"][i]
@@ -715,8 +698,8 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                 mc = model[1]  # tensor cross model component
 
                 summodel = (
-                    productsreal["TpdotTp"][i] * (mp.real ** 2 + mp.imag ** 2)
-                    + productsreal["TcdotTc"][i] * (mc.real ** 2 + mc.imag ** 2)
+                    productsreal["TpdotTp"][i] * (mp.real**2 + mp.imag**2)
+                    + productsreal["TcdotTc"][i] * (mc.real**2 + mc.imag**2)
                     + 2.0
                     * productsreal["TpdotTc"][i]
                     * (mp.real * mc.real + mp.imag * mc.imag)
@@ -737,10 +720,10 @@ class TargetedPulsarLikelihood(bilby.core.likelihood.Likelihood):
                     ml = model[5]
 
                     summodel += (
-                        productsreal["VxdotVx"][i] * (mx.real ** 2 + mx.imag ** 2)
-                        + productsreal["VydotVy"][i] * (my.real ** 2 + my.imag ** 2)
-                        + productsreal["SbdotSb"][i] * (mb.real ** 2 + mb.imag ** 2)
-                        + productsreal["SldotSl"][i] * (ml.real ** 2 + ml.imag ** 2)
+                        productsreal["VxdotVx"][i] * (mx.real**2 + mx.imag**2)
+                        + productsreal["VydotVy"][i] * (my.real**2 + my.imag**2)
+                        + productsreal["SbdotSb"][i] * (mb.real**2 + mb.imag**2)
+                        + productsreal["SldotSl"][i] * (ml.real**2 + ml.imag**2)
                         + 2.0
                         * (
                             productsreal["TpdotVx"][i]
